@@ -7,7 +7,7 @@
  *      implementation of the pqxx::pipeline class
  *   Throughput-optimized query manager
  *
- * Copyright (c) 2003-2006, Jeroen T. Vermeulen <jtv@xs4all.nl>
+ * Copyright (c) 2003-2009, Jeroen T. Vermeulen <jtv@xs4all.nl>
  *
  * See COPYING for copyright license.  If you did not receive a file called
  * COPYING with this source code, please notify the distributor of this mistake,
@@ -20,6 +20,10 @@
 #include "pqxx/dbtransaction"
 #include "pqxx/pipeline"
 
+#include "pqxx/internal/gates/connection-pipeline.hxx"
+#include "pqxx/internal/gates/result-creation.hxx"
+
+
 using namespace PGSTD;
 using namespace pqxx;
 using namespace pqxx::internal;
@@ -46,7 +50,7 @@ pqxx::pipeline::pipeline(transaction_base &t, const PGSTD::string &Name) :
   m_error(qid_limit())
 {
   m_issuedrange = make_pair(m_queries.end(), m_queries.end());
-  register_me();
+  attach();
 }
 
 
@@ -55,13 +59,26 @@ pqxx::pipeline::~pipeline() throw ()
 #ifdef PQXX_QUIET_DESTRUCTORS
   disable_noticer Quiet(m_Trans.conn());
 #endif
-  try { flush(); } catch (const exception &) {}
+  try { cancel(); } catch (const exception &) {}
+  detach();
+}
+
+
+void pqxx::pipeline::attach()
+{
+  if (!registered()) register_me();
+}
+
+
+void pqxx::pipeline::detach()
+{
   if (registered()) unregister_me();
 }
 
 
 pipeline::query_id pqxx::pipeline::insert(const PGSTD::string &q)
 {
+  attach();
   const query_id qid = generate_id();
   pqxxassert(qid > 0);
   pqxxassert(m_queries.lower_bound(qid)==m_queries.end());
@@ -100,6 +117,7 @@ void pqxx::pipeline::complete()
     receive(m_queries.end());
     pqxxassert((m_error!=qid_limit()) || !have_pending());
   }
+  detach();
   pqxxassert((m_num_waiting == 0) || (m_error != qid_limit()));
   pqxxassert(!m_dummy_pending);
 }
@@ -107,12 +125,27 @@ void pqxx::pipeline::complete()
 
 void pqxx::pipeline::flush()
 {
-  if (m_queries.empty()) return;
-  if (have_pending()) receive(m_issuedrange.second);
-  m_issuedrange.first = m_issuedrange.second = m_queries.end();
-  m_num_waiting = 0;
-  m_dummy_pending = false;
-  m_queries.clear();
+  if (!m_queries.empty())
+  {
+    if (have_pending()) receive(m_issuedrange.second);
+    m_issuedrange.first = m_issuedrange.second = m_queries.end();
+    m_num_waiting = 0;
+    m_dummy_pending = false;
+    m_queries.clear();
+  }
+  detach();
+}
+
+
+void pqxx::pipeline::cancel()
+{
+  while (have_pending())
+  {
+    gate::connection_pipeline(m_Trans.conn()).cancel_query();
+    QueryMap::iterator canceled_query = m_issuedrange.first;
+    ++m_issuedrange.first;
+    m_queries.erase(canceled_query);
+  }
 }
 
 
@@ -185,22 +218,23 @@ void pqxx::pipeline::issue()
   if (m_error < qid_limit()) return;
 
   // Start with oldest query (lowest id) not in previous issue range
-  const QueryMap::iterator oldest = m_issuedrange.second;
+  QueryMap::iterator oldest = m_issuedrange.second;
   pqxxassert(oldest != m_queries.end());
 
   // Construct cumulative query string for entire batch
   string cum = separated_list(theSeparator,oldest,m_queries.end(),getquery());
-  const QueryMap::size_type num_issued = distance(oldest,m_queries.end());
+  const QueryMap::size_type num_issued =
+    internal::distance(oldest,m_queries.end());
   const bool prepend_dummy = (num_issued > 1);
   if (prepend_dummy) cum = theDummyQuery + cum;
 
-  m_Trans.start_exec(cum);
+  gate::connection_pipeline(m_Trans.conn()).start_exec(cum);
 
   // Since we managed to send out these queries, update state to reflect this
   m_dummy_pending = prepend_dummy;
   m_issuedrange.first = oldest;
   m_issuedrange.second = m_queries.end();
-  m_num_waiting -= num_issued;
+  m_num_waiting -= int(num_issued);
 }
 
 
@@ -217,7 +251,8 @@ bool pqxx::pipeline::obtain_result(bool expect_none)
   pqxxassert(!m_dummy_pending);
   pqxxassert(!m_queries.empty());
 
-  internal::pq::PGresult *r = m_Trans.get_result();
+  gate::connection_pipeline gate(m_Trans.conn());
+  internal::pq::PGresult *r = gate.get_result();
   if (!r)
   {
     if (have_pending() && !expect_none)
@@ -229,7 +264,11 @@ bool pqxx::pipeline::obtain_result(bool expect_none)
   }
 
   pqxxassert(r);
-  const result res(r, 0, m_queries.begin()->second.get_query());
+  const result res = gate::result_creation::create(
+	r,
+	0,
+	m_queries.begin()->second.get_query(),
+	gate::connection_pipeline(m_Trans.conn()).encoding_code());
 
   if (!have_pending())
   {
@@ -251,17 +290,23 @@ bool pqxx::pipeline::obtain_result(bool expect_none)
 void pqxx::pipeline::obtain_dummy()
 {
   pqxxassert(m_dummy_pending);
-  internal::pq::PGresult *const r = m_Trans.get_result();
+  gate::connection_pipeline gate(m_Trans.conn());
+  internal::pq::PGresult *const r = gate.get_result();
   m_dummy_pending = false;
 
   if (!r)
     internal_error("pipeline got no result from backend when it expected one");
 
-  result R(r, 0, "[DUMMY PIPELINE QUERY]");
+  result R = gate::result_creation::create(
+	r,
+	0,
+	"[DUMMY PIPELINE QUERY]",
+	gate::connection_pipeline(m_Trans.conn()).encoding_code());
+
   bool OK = false;
   try
   {
-    R.CheckStatus();
+    gate::result_creation(R).CheckStatus();
     OK = true;
   }
   catch (const sql_error &)
@@ -296,9 +341,7 @@ void pqxx::pipeline::obtain_dummy()
 
 
   // Reset internal state to forget botched batch attempt
-  // (Not using std::distance() because of problem with SunC++)
-  for (QueryMap::iterator i = m_issuedrange.first; i != stop; ++i)
-    ++m_num_waiting;
+  m_num_waiting += int(internal::distance(m_issuedrange.first, stop));
   m_issuedrange.second = m_issuedrange.first;
 
   pqxxassert(!m_dummy_pending);
@@ -315,7 +358,7 @@ void pqxx::pipeline::obtain_dummy()
       const string &query = m_issuedrange.first->second.get_query();
       const result res(m_Trans.exec(query));
       m_issuedrange.first->second.set_result(res);
-      res.CheckStatus();
+      gate::result_creation(res).CheckStatus();
       ++m_issuedrange.first;
     }
     while (m_issuedrange.first != stop);
@@ -330,7 +373,8 @@ void pqxx::pipeline::obtain_dummy()
     QueryMap::const_iterator q = m_issuedrange.first;
     set_error_at( (q == m_queries.end()) ?  thud + 1 : q->first);
 
-    pqxxassert(m_num_waiting == distance(m_issuedrange.second,m_queries.end()));
+    pqxxassert(m_num_waiting ==
+      internal::distance(m_issuedrange.second, m_queries.end()));
   }
 
   pqxxassert(m_issuedrange.first != m_queries.end());
@@ -352,7 +396,7 @@ pqxx::pipeline::retrieve(pipeline::QueryMap::iterator q)
   if (m_issuedrange.second != m_queries.end() &&
       (q->first >= m_issuedrange.second->first))
   {
-    pqxxassert(distance(m_issuedrange.second, q) >= 0);
+    pqxxassert(internal::distance(m_issuedrange.second, q) >= 0);
 
     if (have_pending()) receive(m_issuedrange.second);
     if (m_error == qid_limit()) issue();
@@ -387,7 +431,7 @@ pqxx::pipeline::retrieve(pipeline::QueryMap::iterator q)
 
   m_queries.erase(q);
 
-  R.CheckStatus();
+  gate::result_creation(R).CheckStatus();
   return P;
 }
 
@@ -395,14 +439,17 @@ pqxx::pipeline::retrieve(pipeline::QueryMap::iterator q)
 void pqxx::pipeline::get_further_available_results()
 {
   pqxxassert(!m_dummy_pending);
-  while (!m_Trans.is_busy() && obtain_result()) m_Trans.consume_input();
+  gate::connection_pipeline gate(m_Trans.conn());
+  while (!gate.is_busy() && obtain_result())
+    if (!gate.consume_input()) throw broken_connection();
 }
 
 
 void pqxx::pipeline::receive_if_available()
 {
-  m_Trans.consume_input();
-  if (m_Trans.is_busy()) return;
+  gate::connection_pipeline gate(m_Trans.conn());
+  if (!gate.consume_input()) throw broken_connection();
+  if (gate.is_busy()) return;
 
   if (m_dummy_pending) obtain_dummy();
   if (have_pending()) get_further_available_results();
@@ -416,7 +463,7 @@ void pqxx::pipeline::receive(pipeline::QueryMap::const_iterator stop)
   if (m_dummy_pending) obtain_dummy();
 
   while (obtain_result() &&
-         QueryMap::const_iterator(m_issuedrange.first) != stop);
+         QueryMap::const_iterator(m_issuedrange.first) != stop) ;
 
   // Also haul in any remaining "targets of opportunity"
   if (QueryMap::const_iterator(m_issuedrange.first) == stop)
