@@ -7,7 +7,7 @@
  *      implementation of the pqxx::tablestream class.
  *   pqxx::tablestream provides optimized batch access to a database table
  *
- * Copyright (c) 2001-2005, Jeroen T. Vermeulen <jtv@xs4all.nl>
+ * Copyright (c) 2001-2006, Jeroen T. Vermeulen <jtv@xs4all.nl>
  *
  * See COPYING for copyright license.  If you did not receive a file called
  * COPYING with this source code, please notify the distributor of this mistake,
@@ -23,7 +23,8 @@
 using namespace PGSTD;
 
 
-pqxx::tablestream::tablestream(transaction_base &STrans, const string &Null) :
+pqxx::tablestream::tablestream(transaction_base &STrans,
+	const PGSTD::string &Null) :
   internal::namedclass("tablestream"),
   internal::transactionfocus(STrans),
   m_Null(Null),
